/*@HEADER
// ***********************************************************************
// 
//       Ifpack2: Tempated Object-Oriented Algebraic Preconditioner Package
//                 Copyright (2009) Sandia Corporation
// 
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
// 
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//  
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//  
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA
// Questions? Contact Michael A. Heroux (maherou@sandia.gov) 
// 
// ***********************************************************************
//@HEADER
*/


#ifndef IFPACK2_BORDEREDOPERATOR_DEF_HPP
#define IFPACK2_BORDEREDOPERATOR_DEF_HPP

#include "Ifpack2_BorderedOperator_decl.hpp"

namespace Ifpack2 {

//Definitions for the BorderedOperator methods:

//==============================================================================
template< class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node >
BorderedOperator<Scalar, LocalOrdinal, GlobalOrdinal, Node >::
BorderedOperator(const Teuchos::RCP<const Tpetra::Operator<Scalar, LocalOrdinal, GlobalOrdinal, Node > >& A) : 
  A_(A)
{ 
  TEUCHOS_TEST_FOR_EXCEPTION(A_ == Teuchos::null, std::runtime_error, 
      Teuchos::typeName(*this) << "::BordredOperator(): input matrix reference was null.");
}
//==============================================================================
//template<class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node >
  //BorderedOperator<Scalar, LocalOrdinal, GlobalOrdinal, Node>::~BorderedOperator(){
//}
//==============================================================================
template< class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node >
const Teuchos::RCP<const Tpetra::Map<LocalOrdinal,GlobalOrdinal,Node> >& 
BorderedOperator<Scalar, LocalOrdinal, GlobalOrdinal, Node >::getDomainMap() const
{
  return A_->getDomainMap();
}
//==============================================================================
template< class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node >
const Teuchos::RCP<const Tpetra::Map<LocalOrdinal,GlobalOrdinal,Node> >& 
BorderedOperator<Scalar, LocalOrdinal, GlobalOrdinal, Node >::getRangeMap() const
{
  return A_->getRangeMap();
}
//==============================================================================
template< class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node >
bool
BorderedOperator<Scalar, LocalOrdinal, GlobalOrdinal, Node >::hasTransposeApply() const 
{
  return A_->hasTransposeApply();
}
//==============================================================================
template< class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node >
void BorderedOperator<Scalar, LocalOrdinal, GlobalOrdinal, Node >::apply(
     const Tpetra::MultiVector<Scalar, LocalOrdinal, GlobalOrdinal, Node >& X,
           Tpetra::MultiVector<Scalar, LocalOrdinal, GlobalOrdinal, Node >& Y,
     Teuchos::ETransp mode, 
     Scalar coefAx, 
     Scalar coefY ) const 
{
  //bool opHasTrans = A_->hasTransposeApply();
  //TEUCHOS_TEST_FOR_EXCEPTION( mode  &&  !opHasTrans, std::runtime_error,
  //"Ifpack2::BorderedOperator::apply() ERROR: The operator does not implement transpose.");
  TEUCHOS_TEST_FOR_EXCEPTION(X.getNumVectors() != Y.getNumVectors(), std::runtime_error,
     "Ifpack2::BorderedOperator::apply() ERROR: X.getNumVectors() != Y.getNumVectors().");
  A_->apply(X, Y, mode, coefAx, coefY );
}


}//namespace Ifpack2

#endif /* IFPACK2_BorderedOperator_DEF_HPP */

